#!/bin/bash
###########################################################################
#   Copyright (C) 2023  by Stephen Lyons - slysven@virginmedia.com        #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
###########################################################################

# Version: 1.0.0

# Script to run once in a directory such as ~/src in a MSYS2 shell to
# install as much as possible to be able to develop 64/32 Bit Windows
# version of Mudlet

# To be used prior to building Mudlet, after that run:
# * build-mudlet-for-window.sh to compile the currently checked out code
#   (defaults to the 'development' branch
# * package-mudlet-for-windows.sh to put everything together in an archive that
#   can be unzipped and run on a different PC.

# Exit codes:
# 0 - Everything is fine. 8-)
# 1 - Failure to change to a directory
# 2 - Unsupported MSYS2/MINGGW shell type

if [ "${MSYSTEM}" = "MINGW64" ]; then
  export BUILD_BITNESS="64"
  export BUILDCOMPONENT="x86_64"
elif [ "${MSYSTEM}" = "MINGW32" ]; then
  export BUILD_BITNESS="32"
  export BUILDCOMPONENT="i686"
elif [ "${MSYSTEM}" = "MSYS" ]; then
  echo "Please run this script from an MINGW32 or MINGW64 type bash terminal appropriate"
  echo "to the bitness you want to work on. You may do this once for each of them should"
  echo "you wish to do both."
  exit 2
else
  echo "This script is not set up to handle systems of type ${MSYSTEM}, only MINGW32 or"
  echo "MINGW64 are currently supported. Please rerun this in a bash terminal of one"
  echo "of those two types."
  exit 2
fi

# We use this internally - but it is actually the same as 
export MINGW_BASE_DIR="C:/msys64/mingw${BUILD_BITNESS}"
# A more compact - but not necessarily understood by other than MSYS/MINGW
# executables - path:
export MINGW_INTERNAL_BASE_DIR="/mingw${BUILD_BITNESS}"
#
# FIXME: don't add duplicates but rearrange instead to put them in the "right" order:
#
export PATH="${MINGW_INTERNAL_BASE_DIR}/usr/local/bin:${MINGW_INTERNAL_BASE_DIR}/bin:/usr/bin:${PATH}"
echo "MSYSTEM is: ${MSYSTEM}"
echo "PATH is now: ${PATH}"
echo ""

# Options to consider:
# --Sy = Sync, refresh as well as installing the specified packages
# --noconfirm = do not ask for user intervention
# --noprogressbar = do not show progress bars as they are not useful in scripts
echo "Updating and installing ${MSYSTEM} packages..."
echo ""
echo "This will take a long time, so feel free to go and have a cup of tea"
echo " (other beverages are available)...!"
echo ""
/usr/bin/pacman -S --needed --noconfirm \
  git \
  man \
  rsync \
  "mingw-w64-${BUILDCOMPONENT}-toolchain" \
  "mingw-w64-${BUILDCOMPONENT}-qt5" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-base-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-gamepad-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-multimedia-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-svg-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-speech-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-imageformats-debug" \
  "mingw-w64-${BUILDCOMPONENT}-qt5-winextras-debug" \
  "mingw-w64-${BUILDCOMPONENT}-pcre" \
  "mingw-w64-${BUILDCOMPONENT}-libzip" \
  "mingw-w64-${BUILDCOMPONENT}-ntldd" \
  "mingw-w64-${BUILDCOMPONENT}-pugixml" \
  "mingw-w64-${BUILDCOMPONENT}-lua51" \
  "mingw-w64-${BUILDCOMPONENT}-lua51-lpeg" \
  "mingw-w64-${BUILDCOMPONENT}-lua51-lsqlite3" \
  "mingw-w64-${BUILDCOMPONENT}-hunspell" \
  "mingw-w64-${BUILDCOMPONENT}-zlib" \
  "mingw-w64-${BUILDCOMPONENT}-boost" \
  "mingw-w64-${BUILDCOMPONENT}-yajl" \
  "mingw-w64-${BUILDCOMPONENT}-SDL2" \
  "mingw-w64-${BUILDCOMPONENT}-lua-luarocks"

# Optional extras for development:
#  "mingw-w64-${BUILDCOMPONENT}-ccache \
#  "mingw-w64-${BUILDCOMPONENT}-qt-creator \
#  "mingw-w64-${BUILDCOMPONENT}-qt5-doc \
#  "mingw-w64-${BUILDCOMPONENT}-cmake \
#  "mingw-w64-${BUILDCOMPONENT}-clang \
#  "mingw-w64-${BUILDCOMPONENT}-ninja

echo ""
# This package is not available for the 32-bit case:
echo "  Additional packages for ${MSYSTEM}:"
if [ "${MSYSTEM}" = "MINGW64" ]; then
  /usr/bin/pacman -S --needed --noconfirm "mingw-w64-${BUILDCOMPONENT}-qtkeychain-qt5"
else
  "  None"
fi

echo ""
echo "Tweaking location for constructed Luarocks so 32 and 64 bits ones do not"
echo "end up in the same place."
echo ""

cp "${MINGW_INTERNAL_BASE_DIR}/etc/luarocks/config-5.1.lua" "${MINGW_INTERNAL_BASE_DIR}/etc/luarocks/config-5.1.lua.orig"
/usr/bin/sed "s|.. \"/.luarocks\"|.. \"/.luarocks-${MSYSTEM}\"|" "${MINGW_INTERNAL_BASE_DIR}/etc/luarocks/config-5.1.lua.orig" > "${MINGW_INTERNAL_BASE_DIR}/etc/luarocks/config-5.1.lua"

# Need to overcome a problem with luarock 3.9.0 which uses Windows CMD MKDIR
# but which cannot make any missing intermediate directories if the
# luafilesystem module for Lua 5.4 is not present, see:
# https://github.com/msys2/MINGW-packages/pull/12002
#
# CHECK: This may be locale specific - the text "failed making directory" may
# be translated:
neededPath=$(${MINGW_INTERNAL_BASE_DIR}/bin/luarocks --lua-version 5.4 install luafilesystem 2>&1 | grep "failed making directory" | cut -c 32-)
until [ -z "${neededPath}" ]; do
  echo "Inserting a needed directory: ${neededPath} ..."
  mkdir -p "${neededPath}"
  neededPath=$(${MINGW_INTERNAL_BASE_DIR}/bin/luarocks --lua-version 5.4 install luafilesystem 2>&1 | grep "failed making directory" | cut -c 32-)
done
# Doing the above fixes things for 5.1 luarocks subsequently (as luarocks
# itself runs in a Lua 5.4 environment - otherwise one has to do the same thing
# for EVERY luarock!

# Save the wanted Luarocks 5.1 command so it can be used repeatedly next
# this uses the "system" (shared between all users) tree, using the --local
# option does NOT work, but the alternative --tree "user" (the literal string
# user NOT the user's name) does for "per user" luarocks as the previous use
# of sed above has "fixed" things:
ROCKCOMMAND="${MINGW_INTERNAL_BASE_DIR}/bin/luarocks --lua-version 5.1"

# FIXME: Only install if needed - this whole script is safe to be rerun
# (idempotent) but it is an unnecessary delay to reinstall the same things over
# again:
echo ""
echo "  Installing needed luarocks..."
echo ""
echo "    luafilesystem"
echo ""
${ROCKCOMMAND} install luafilesystem
echo ""
echo "    lua-yajl"
echo ""
${ROCKCOMMAND} install lua-yajl
echo ""
echo "    luautf8"
echo ""
${ROCKCOMMAND} install luautf8
echo ""
echo "    lua-zip"
echo ""
${ROCKCOMMAND} install lua-zip
echo ""
echo "    lrexlib-pcre"
echo ""
${ROCKCOMMAND} install lrexlib-pcre
echo ""
echo "    luasql-sqlite3"
echo ""
${ROCKCOMMAND} install luasql-sqlite3
echo ""
echo "    ... luarocks installation done"
echo ""
echo "  Making a ${HOME}/src directory (if it does not exist)"
echo "    - there that a local Mudlet git repository for you to build and work"
echo "    on the project..."
echo ""
mkdir -p ~/src
cd src || exit 1
if [ ! -d "./mudlet" ]; then
    echo "    Cloning the Mudlet project's source code..."
    echo ""
    git clone https://github.com/Mudlet/Mudlet.git mudlet
else
    echo "    There is already a ${HOME}/src/mudlet"
    echo "    directory so it seems that there is no need to clone the Mudlet"
    echo "    project's source code..."
    echo ""
fi
echo ""
echo "    ... done."
echo ""
cd ~ || exit 1
echo "  ... setup-windows-sdk.sh shell script finished."
echo ""

exit 0
